@@ -25,9 +25,6 @@ GType arv_auto_get_type	(void) G_GNUC_CONST;
 #define ARV_TYPE_ACQUISITION_MODE	(arv_acquisition_mode_get_type())
 GType arv_acquisition_mode_get_type	(void) G_GNUC_CONST;
 
-#define ARV_TYPE_PIXEL_FORMAT	(arv_pixel_format_get_type())
-GType arv_pixel_format_get_type	(void) G_GNUC_CONST;
-
 /* Enumerations from "arvdebug.h" */
 
 #define ARV_TYPE_DEBUG_LEVEL	(arv_debug_level_get_type())
@@ -51,6 +48,11 @@ GType arv_gc_sign_get_type	(void) G_GNUC_CONST;
 #define ARV_TYPE_GC_REGISTER_NODE_TYPE	(arv_gc_register_node_type_get_type())
 GType arv_gc_register_node_type_get_type	(void) G_GNUC_CONST;
 
+/* Enumerations from "arvdevice.h" */
+
+#define ARV_TYPE_DEVICE_STATUS	(arv_device_status_get_type())
+GType arv_device_status_get_type	(void) G_GNUC_CONST;
+
 /* Enumerations from "arvstream.h" */
 
 #define ARV_TYPE_STREAM_CALLBACK_TYPE	(arv_stream_callback_type_get_type())
