@@ -93,13 +93,24 @@ G_BEGIN_DECLS
 #define ARV_GVBS_N_STREAM_CHANNELS_OFFSET		0x00000904
 
 #define ARV_GVBS_GVCP_CAPABILITY_OFFSET			0x00000934
-#define ARV_GVBS_GVCP_CAPABILITY_CONCATENATION		1 << 0
-#define ARV_GVBS_GVCP_CAPABILITY_WRITE_MEMORY		1 << 1
-#define ARV_GVBS_GVCP_CAPABILITY_PACKET_RESEND		1 << 2
-#define ARV_GVBS_GVCP_CAPABILITY_EVENT			1 << 3
-#define ARV_GVBS_GVCP_CAPABILITY_EVENT_DATA		1 << 4
-#define ARV_GVBS_GVCP_CAPABILITY_SERIAL_NUMBER		1 << 30
-#define ARV_GVBS_GVCP_CAPABILITY_NAME_REGISTER		1 << 31
+#define ARV_GVBS_GVCP_CAPABILITY_CONCATENATION			1 << 0
+#define ARV_GVBS_GVCP_CAPABILITY_WRITE_MEMORY			1 << 1
+#define ARV_GVBS_GVCP_CAPABILITY_PACKET_RESEND			1 << 2
+#define ARV_GVBS_GVCP_CAPABILITY_EVENT				1 << 3
+#define ARV_GVBS_GVCP_CAPABILITY_EVENT_DATA			1 << 4
+#define ARV_GVBS_GVCP_CAPABILITY_PENDING_ACK			1 << 5
+#define ARV_GVBS_GVCP_CAPABILITY_ACTION				1 << 6
+#define ARV_GVBS_GVCP_CAPABILITY_PRIMARY_APPLICATION_SWITCHOVER 1 << 21
+#define ARV_GVBS_GVCP_CAPABILITY_EXTENDED_STATUS_CODES		1 << 22
+#define ARV_GVBS_GVCP_CAPABILITY_DISCOVERY_ACK_DELAY_WRITABLE	1 << 23
+#define ARV_GVBS_GVCP_CAPABILITY_DISCOVERY_ACK_DELAY		1 << 24
+#define ARV_GVBS_GVCP_CAPABILITY_TEST_DATA			1 << 25
+#define ARV_GVBS_GVCP_CAPABILITY_MANIFEST_TABLE			1 << 26
+#define ARV_GVBS_GVCP_CAPABILITY_CCP_APPLICATION_SOCKET 	1 << 27
+#define ARV_GVBS_GVCP_CAPABILITY_LINK_SPEED			1 << 28
+#define ARV_GVBS_GVCP_CAPABILITY_HEARTBEAT_DISABLE		1 << 29
+#define ARV_GVBS_GVCP_CAPABILITY_SERIAL_NUMBER			1 << 30
+#define ARV_GVBS_GVCP_CAPABILITY_NAME_REGISTER			1 << 31
 
 #define ARV_GVBS_HEARTBEAT_TIMEOUT_OFFSET		0x00000938
 #define ARV_GVBS_TIMESTAMP_TICK_FREQUENCY_HIGH_OFFSET	0x0000093c
