@@ -25,8 +25,6 @@
  * @short_description: Gigabit ethernet camera device
  */
 
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <arvgvdevice.h>
 #include <arvgc.h>
 #include <arvdebug.h>
@@ -35,6 +33,7 @@
 #include <arvgvsp.h>
 #include <arvzip.h>
 #include <arvstr.h>
+#include <arvmisc.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -43,7 +42,11 @@ static GObjectClass *parent_class = NULL;
 /* Shared data (main thread - heartbeat) */
 
 typedef struct {
+#if GLIB_CHECK_VERSION(2,32,0)
+	GMutex mutex;
+#else
 	GMutex *mutex;
+#endif
 
 	guint16 packet_id;
 
@@ -71,6 +74,9 @@ struct _ArvGvDevicePrivate {
 
 	char *genicam_xml;
 	size_t genicam_xml_size;
+
+	gboolean is_packet_resend_supported;
+	gboolean is_write_memory_supported;
 };
 
 GRegex *
@@ -99,7 +105,11 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 
 	g_return_val_if_fail (answer_size <= ARV_GV_DEVICE_BUFFER_SIZE, FALSE);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_lock (&io_data->mutex);
+#else
 	g_mutex_lock (io_data->mutex);
+#endif
 
 	packet = arv_gvcp_packet_new_read_memory_cmd (address,
 						      ((size + sizeof (guint32) - 1)
@@ -147,7 +157,11 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 
 	arv_gvcp_packet_free (packet);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_unlock (&io_data->mutex);
+#else
 	g_mutex_unlock (io_data->mutex);
+#endif
 
 	if (!success) {
 		if (error != NULL && *error == NULL)
@@ -167,7 +181,11 @@ _write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_lock (&io_data->mutex);
+#else
 	g_mutex_lock (io_data->mutex);
+#endif
 
 	packet = arv_gvcp_packet_new_write_memory_cmd (address,
 						       ((size + sizeof (guint32) - 1) /
@@ -216,7 +234,11 @@ _write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *
 
 	arv_gvcp_packet_free (packet);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_unlock (&io_data->mutex);
+#else
 	g_mutex_unlock (io_data->mutex);
+#endif
 
 	if (!success) {
 		if (error != NULL && *error == NULL)
@@ -236,7 +258,11 @@ _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_plac
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_lock (&io_data->mutex);
+#else
 	g_mutex_lock (io_data->mutex);
+#endif
 
 	packet = arv_gvcp_packet_new_read_register_cmd (address, 0, &packet_size);
 
@@ -281,7 +307,11 @@ _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_plac
 
 	arv_gvcp_packet_free (packet);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_unlock (&io_data->mutex);
+#else
 	g_mutex_unlock (io_data->mutex);
+#endif
 
 	if (!success) {
 		*value_placeholder = 0;
@@ -303,7 +333,11 @@ _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value, GEr
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_lock (&io_data->mutex);
+#else
 	g_mutex_lock (io_data->mutex);
+#endif
 
 	packet = arv_gvcp_packet_new_write_register_cmd (address, value, io_data->packet_id, &packet_size);
 
@@ -348,7 +382,11 @@ _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value, GEr
 
 	arv_gvcp_packet_free (packet);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_unlock (&io_data->mutex);
+#else
 	g_mutex_unlock (io_data->mutex);
+#endif
 
 	if (!success) {
 		if (error != NULL && *error == NULL)
@@ -733,7 +771,7 @@ arv_gv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void
 	stream_port = arv_gv_stream_get_port (ARV_GV_STREAM (stream));
 
 	if (!arv_device_write_register (device, ARV_GVBS_STREAM_CHANNEL_0_IP_ADDRESS_OFFSET,
-				   g_htonl(*((guint32 *) address_bytes)), NULL) ||
+					g_htonl(*((guint32 *) address_bytes)), NULL) ||
 	    !arv_device_write_register (device, ARV_GVBS_STREAM_CHANNEL_0_PORT_OFFSET, stream_port, NULL)) {
 		arv_warning_device ("[GvDevice::create_stream] Stream configuration failed");
 
@@ -741,6 +779,9 @@ arv_gv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void
 		return NULL;
 	}
 
+	if (!gv_device->priv->is_packet_resend_supported)
+		g_object_set (stream, "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL); 
+
 	arv_debug_device ("[GvDevice::create_stream] Stream port = %d", stream_port);
 
 	return stream;
@@ -765,7 +806,7 @@ arv_gv_device_read_memory (ArvDevice *device, guint32 address, guint32 size, voi
 		block_size = MIN (ARV_GVCP_DATA_SIZE_MAX, size - i * ARV_GVCP_DATA_SIZE_MAX);
 		if (!_read_memory (gv_device->priv->io_data,
 				   address + i * ARV_GVCP_DATA_SIZE_MAX,
-				   block_size, buffer + i * ARV_GVCP_DATA_SIZE_MAX, error))
+				   block_size, ((char *) buffer) + i * ARV_GVCP_DATA_SIZE_MAX, error))
 			return FALSE;
 	}
 
@@ -783,7 +824,7 @@ arv_gv_device_write_memory (ArvDevice *device, guint32 address, guint32 size, vo
 		block_size = MIN (ARV_GVCP_DATA_SIZE_MAX, size - i * ARV_GVCP_DATA_SIZE_MAX);
 		if (!_write_memory (gv_device->priv->io_data,
 				    address + i * ARV_GVCP_DATA_SIZE_MAX,
-				    block_size, buffer + i * ARV_GVCP_DATA_SIZE_MAX, error))
+				    block_size, ((char *) buffer) + i * ARV_GVCP_DATA_SIZE_MAX, error))
 			return FALSE;
 	}
 
@@ -813,6 +854,7 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 	ArvGvDeviceIOData *io_data;
 	ArvGvDeviceHeartbeatData *heartbeat_data;
 	char *address_string;
+	guint32 capabilities;
 
 	g_return_val_if_fail (G_IS_INET_ADDRESS (interface_address), NULL);
 	g_return_val_if_fail (G_IS_INET_ADDRESS (device_address), NULL);
@@ -828,7 +870,11 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 
 	io_data = g_new0 (ArvGvDeviceIOData, 1);
 
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_init (&io_data->mutex);
+#else
 	io_data->mutex = g_mutex_new ();
+#endif
 	io_data->packet_id = 65300; /* Start near the end of the circular counter */
 
 	io_data->interface_address = g_inet_socket_address_new (interface_address, 0);
@@ -859,9 +905,15 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 
 	gv_device->priv->heartbeat_data = heartbeat_data;
 
-	gv_device->priv->heartbeat_thread = g_thread_create (arv_gv_device_heartbeat_thread,
-							     gv_device->priv->heartbeat_data,
-							     TRUE, NULL);
+	gv_device->priv->heartbeat_thread = arv_g_thread_new ("arv_gv_heartbeat", arv_gv_device_heartbeat_thread,
+							      gv_device->priv->heartbeat_data);
+
+	arv_device_read_register (ARV_DEVICE (gv_device), ARV_GVBS_GVCP_CAPABILITY_OFFSET, &capabilities, NULL);
+	gv_device->priv->is_packet_resend_supported = (capabilities & ARV_GVBS_GVCP_CAPABILITY_PACKET_RESEND) != 0;
+	gv_device->priv->is_write_memory_supported = (capabilities & ARV_GVBS_GVCP_CAPABILITY_WRITE_MEMORY) != 0;
+
+	arv_debug_device ("[GvDevice::new] Packet resend = %s", gv_device->priv->is_packet_resend_supported ? "yes" : "no");
+	arv_debug_device ("[GvDevice::new] Write memory = %s", gv_device->priv->is_write_memory_supported ? "yes" : "no");
 
 	return ARV_DEVICE (gv_device);
 }
@@ -902,7 +954,11 @@ arv_gv_device_finalize (GObject *object)
 	g_object_unref (io_data->interface_address);
 	g_object_unref (io_data->socket);
 	g_free (io_data->buffer);
+#if GLIB_CHECK_VERSION(2,32,0)
+	g_mutex_clear (&io_data->mutex);
+#else
 	g_mutex_free (io_data->mutex);
+#endif
 
 	g_free (gv_device->priv->io_data);
 
