@@ -20,8 +20,6 @@
  * Author: Emmanuel Pacaud <emmanuel@gnome.org>
  */
 
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <arvconfig.h>
 #include <gtk/gtk.h>
 #include <gst/gst.h>
@@ -173,7 +171,7 @@ arv_viewer_new_buffer_cb (ArvStream *stream, ArvViewer *viewer)
 			data = g_malloc (size);	
 
 			for (i = 0; i < arv_buffer->height; i++)
-				memcpy (data + i * gst_row_stride, arv_buffer->data + i * arv_row_stride, arv_row_stride);
+				memcpy (((char *) data) + i * gst_row_stride, ((char *) arv_buffer->data) + i * arv_row_stride, arv_row_stride);
 
 			GST_BUFFER_DATA (buffer) = data;
 			GST_BUFFER_MALLOCDATA (buffer) = data;
@@ -830,7 +828,7 @@ main (int argc,char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	g_thread_init (NULL);
+	arv_g_thread_init (NULL);
 
 	context = g_option_context_new (NULL);
 	g_option_context_add_main_entries (context, arv_viewer_option_entries, NULL);
